//    This file is part of Motion Watch.

//    Motion Watch is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    Motion Watch is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.

#include "mo_detect.h"

void detectMoInStream(const string &bufPath, shared_t *share)
{
    ifstream fileIn(bufPath);
    string   tsPath;
    Mat      thumbnail;

    auto clipPathFilter = genTimeStr("VIDEO_TS/live/%Y/%j/%H/");

    for (string line; getline(fileIn, line); )
    {
        if (line.starts_with(clipPathFilter))
        {
            tsPath = line;
        }
    }

    if (!tsPath.empty())
    {
        if (moDetect(tsPath, thumbnail, share))
        {
            auto eventName = genTimeStr("%Y-%j-%H-%M");

            if (share->recList.find(eventName) != share->recList.end())
            {
                share->recList[eventName].srcPaths.push_back(tsPath);
            }
            else
            {
                pls_t event;

                event.srcPaths.push_back(tsPath);

                event.createTime = genEpoch();
                event.thumbnail  = thumbnail.clone();
                event.evName     = eventName;

                share->recList.insert(pair{eventName, event});
            }

            share->skipCmd = true;
        }
    }
}

bool imgDiff(Mat &prev, Mat &next, shared_t *share)
{
    auto ret = false;

    cvtColor(prev, prev, COLOR_BGR2GRAY);
    cvtColor(next, next, COLOR_BGR2GRAY);

    Mat diff;

    absdiff(prev, next, diff);
    threshold(diff, diff, share->pixThresh, 255, THRESH_BINARY);

    auto diffScore = countNonZero(diff);

    detLog("diff_score: " + to_string(diffScore), share);

    return diffScore >= share->imgThresh;
}

bool moDetect(const string &buffFile, Mat &vidThumb, shared_t *share)
{
    auto mod = false;

    detLog("stream_clip: " + buffFile, share);

    VideoCapture capture(buffFile.c_str(), CAP_FFMPEG);

    if (capture.isOpened())
    {
        Mat  prev;
        Mat  next;

        detLog("capture open successful.", share);

        while (capture.grab())
        {
            if (prev.empty())
            {
                capture.retrieve(prev);
            }
            else
            {
                capture.retrieve(next);

                if (!share->postCmdRunning)
                {
                    if (imgDiff(prev, next, share))
                    {
                        resize(next, vidThumb, Size(720, 480), INTER_LINEAR);

                        mod = true; break;
                    }
                }

                prev.release();
                next.release();
            }

            sleep(1);
        }
    }
    else
    {
        detLog("capture open failure, check debug output.", share);
    }

    capture.release();

    return mod;
}

void wrOutVod(const pls_t &event, shared_t *share)
{
    auto concat = event.evName + ".tmp";

    ofstream file(concat.c_str());

    for (auto i = 0; i < event.srcPaths.size(); ++i)
    {
        file << "file '" << event.srcPaths[i] << "''" << endl;
    }

    file.close();

    system(string("ffmpeg -f concat -safe 0 -i " + concat + " -c copy " + event.evName + ".mp4").c_str());
    remove(concat);
}
